"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.storeInjector = storeInjector;

var _redux = require("redux");

function storeInjector() {
  var loaders = new Map();
  var injected = {};
  var staticReducers = {};
  var store;

  var dispatch = action => store.dispatch(action);

  var setStore = newStore => store = newStore;

  var saveStatic = reducers => {
    staticReducers = reducers;
  };

  var getRootReducer = () => (0, _redux.combineReducers)(Object.assign({}, staticReducers, injected));

  var replaceReducer = () => {
    var rootReducer = getRootReducer();
    store.replaceReducer(rootReducer);
  };
  /**
   * Add reducer to store dynamically.
   *
   * Accepts reducer's name and it's loader - async function, which returns reducer itself
   *
   * Also, function correctly handles some typical cases:
   *  - Resolves default export in case of es6 modules import
   *  - Transforms object with reducers into plain reducer function (therefore you can import reducers without calling combineReducers by yourself)
   *
   * Returns boolean, which shows were reducer been imported or taken from the cache
   *
   *
   * @exports
   * @param {string} name
   * @param {() => Promise<StateReducer>} reducerThunk
   * @returns {Promise<boolean>}
   */


  async function importReducer(name, reducerThunk) {
    if (loaders.has(name)) return true;
    var reducer = await thunkLoader(reducerThunk);
    injected[name] = reducer;
    loaders.set(name, reducerThunk);
    replaceReducer();
    return false;
  }

  async function reload(reducers) {
    saveStatic(reducers);
    injected = {};

    for (var [key, thunk] of loaders.entries()) {
      var reducer = await thunkLoader(thunk);
      injected[key] = reducer;
    }

    replaceReducer();
  }

  async function thunkLoader(thunk) {
    var loaded = await thunk();
    if (typeof loaded === 'function') return loaded;

    if (typeof loaded !== 'object' || loaded == null) {
      var _message = wrongTypeError(loaded);

      throw new TypeError(_message);
    }

    var def = loaded.default;
    if (typeof def === 'function') return def;
    if (def == null) return (0, _redux.combineReducers)(loaded);
    if (typeof def === 'object') return (0, _redux.combineReducers)(def);
    var message = wrongTypeError(def);
    throw new TypeError(message);
  }

  return {
    dispatch,
    setStore,
    saveStatic,
    getRootReducer,
    importReducer,
    reload
  };
}

var wrongTypeError = loaded => `
Thunk must be object with reducers or reducer itself;
got ${typeof loaded} ${String(loaded)}
`;
//# sourceMappingURL=inject-reducer.js.map